function big_smooth,data,width,medsmooth=medsmooth,noccd=noccd

;+
;NAME:
;       big_smooth
;SAMPLE CALLING SEQUENCE:
;       out = big_smooth(data, 15)  ; Boxcar averaged output.
;	out = big_smooth(data, /medsmooth) ; Median smoothed output.
;PURPOSE:
;	To permit "boxcar" smoothing of an vector or an 
;	image without producing the irritating
;	unsmoothed border at the boundaries.
;
;       This program forms an array bigger than the original
;       array by half of the smoothing width on each side and fills
;       the corners with valid average "near edge" data and then smooths
;       the extended array.  It then plucks out and array the size of
;       the original out of the smoothed big array.
;INPUT:
;       data = the 1-d or 2-d array to be smoothed
;OPTIONAL INPUT:
;	width = the number of pixels over which to smooth.
;		[default=3]
;OPTIONAL KEYWORD INPUT
;	/medsmooth forms a median array rather than an average array.
;	/noccd is for a normal image -- with "edge problems" of the CCD
;OUTPUTS:
;       out = the smoothed array of same type as input array
;	    (Note:  Actual smoothing is done in floating point.)
;RESTRICTIONS:
;       Use only on a single array at a time, not a data cube.
;	Edge treatment special for SXT case, otherwise use /noccd.
;HISTORY:
;       Written by L. Acton on 2/22/94.
;	Corrected indexing errors, 2/23/94.  LWA
;	Changed to extend the image with average data,
;	  avoiding the "problem columns" at the edge.  LWA 3/20/94
;	Modified to handle 1-d smoothing.  LWA 3/20/94
;       Added median smoothing option and default width.  LWA 10/20/96
;	Improved edge correction.  LWA 10/21/96
;	Added /noccd keyword.  LWA 10/27/96
;	Included /nan keyword in call to smooth.  LWA 6/6/04
;-


if NOT keyword_set(width) then wid=3 else wid=width
array=reform(data)

siz=size(array)
x=siz(1)
y=siz(2)
xx=x-1
yy=y-1

w=wid/2
ww=wid/2+1

case 1 of
   (siz(0) eq 1) : begin  ; ONE DIMENSIONAL VECTOR
      big=fltarr(x+2*ww)
      sizb=size(big)
      xb=sizb(1)-1
      big(ww:ww+xx)=array
      for i=0,w do big(i)=total(float(array(0:0+w)))/ww
      for i=0,w do big(i+ww+xx)=total(float(array(xx-w:xx)))/ww
      if NOT keyword_set(medsmooth) then begin
         smth_array=smooth(big,wid,/nan) 
      endif else begin
         smth_array=median(big,wid)
      endelse
      out=smth_array(ww:ww+xx)
   end
   
   (siz(0) eq 2) : begin  ; TWO DIMENSIONAL IMAGE
      big=fltarr(x+2*ww,y+2*ww)
      sizb=size(big)
      xb=sizb(1)-1
      yb=sizb(2)-1

      if keyword_set(noccd) then begin  ;CCD IMAGE OR NOT?
         ll=0 & rr=1 & bb=0 & tt=1
      endif else begin
         ll=2 & rr=0 & bb=7 & tt=0
      endelse

      ;  Put image in center of BIG
      big(ww:ww+xx,ww:ww+yy)=array
      ;  Extend LH side with an average value,
      ;   offset to avoid "problem columns" at edge of CCD.
      for i=0,w+ll do big(i,ww:ww+yy)=total(float(array(ll:ll+w,*)),1)/ww
      ;  Likewise, extend RH side
      for i=0,w+1-rr do $
        big(i+ww+xx,ww:ww+yy)=total(float(array(xx-ww+rr:xx-1+rr,*)),1)/ww
      ;  Extend bottom
      for i=0,w+bb do big(*,i)=total(big(*,bb+ww:bb+w+ww),2)/ww
      ;  Extend top
      for i=0,w do big(*,i+ww+y)=total(big(*,yy+tt:yy+ww-1+tt),2)/ww

      if NOT keyword_set(medsmooth) then begin
         smth_array=smooth(big,wid,/nan) 
      endif else begin
         smth_array=median(big,wid)
      endelse
      out=smth_array(ww:ww+xx,ww:ww+yy)
   end
endcase

;  Set type of output to agree with input.
if (siz(0) eq 2) then ii=3 else ii=2
if (siz(ii) eq 1) then out=byte(out)
if (siz(ii) eq 2) then out=fix(out)
if (siz(ii) eq 3) then out=long(out)

return,out

end

